<?php

namespace App\Http\Controllers;

use App\Models\Address;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\ValidationException;
use App\Exceptions\AddressNotFoundException;

class AddressController extends Controller
{
    public function index()
    {
        try {
            $user = Auth::user();
            $addresses = $user->addresses;

            if ($addresses->isEmpty()) {
                return response()->json([
                    'success' => false,
                    'message' => 'No addresses found'
                ], 404);
            }

            return response()->json(['success' => true, 'addresses' => $addresses]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function store(Request $request)
    {
        try {
            $validated = $request->validate([
                'name' => 'required|string|max:255',
                'address' => 'required|string|max:255',
                'latitude' => 'required|numeric',
                'longitude' => 'required|numeric',
            ]);

            $address = Address::create([
                'user_id' => Auth::id(),
                'name' => $validated['name'],
                'address' => $validated['address'],
                'latitude' => $validated['latitude'],
                'longitude' => $validated['longitude'],
            ]);

            return response()->json(['success' => true, 'address' => $address], 201);
        } catch (ValidationException $e) {
            return response()->json(['success' => false, 'message' => $e->errors()], 422);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $id)
    {
        try {
            $address = Address::findOrFail($id);

            $validated = $request->validate([
                'name' => 'sometimes|required|string|max:255',
                'address' => 'sometimes|required|string|max:255',
                'latitude' => 'sometimes|required|numeric',
                'longitude' => 'sometimes|required|numeric',
            ]);

            $address->update($validated);

            return response()->json(['success' => true, 'address' => $address]);
        } catch (ValidationException $e) {
            return response()->json(['success' => false, 'message' => $e->errors()], 422);
        } catch (ModelNotFoundException $e) {
            throw new AddressNotFoundException();
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function destroy($id)
    {
        try {
            $address = Address::findOrFail($id);
            $address->delete();

            return response()->json(['success' => true, 'message' => 'Address deleted successfully']);
        } catch (ModelNotFoundException $e) {
            throw new AddressNotFoundException();
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }
}
