<?php

namespace App\Http\Controllers;

use App\Models\Delivery;
use App\Models\DeliveryLocation;
use App\Models\DeliveryNotification;
use App\Models\Order;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Laravel\Sanctum\PersonalAccessToken;

class DeliveryController extends Controller
{
    public function login(Request $request)
    {
        try {
            $credentials = $request->only('email', 'password');

            $user = Delivery::where('email', $credentials['email'])->first();

            if (!$user || !Hash::check($credentials['password'], $user->password)) {
                return response()->json(['success' => false, 'message' => 'Invalid credentials'], 401);
            }

            $token = $user->createToken('API Token')->plainTextToken;

            return response()->json(['success' => true, 'token' => $token, 'user' => $user], 200);

        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function register(Request $request)
    {
        try {
            $validated = $request->validate([
                'name' => 'required|string|max:255',
                'email' => 'required|string|email|max:255|unique:deliveries',
                'phone_number' => 'required|string|max:15|unique:deliveries',
                'password' => 'required|string|min:6',
            ]);

            $delivery = Delivery::create($validated);

            $token = $delivery->createToken('API Token')->plainTextToken;

            return response()->json(['success' => true, 'token' => $token, 'user' => $delivery], 201);
        } catch (ValidationException $e) {
            return response()->json(['success' => false, 'message' => $e->errors()], 422);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function profile(Request $request)
    {
        try {
            $user = Auth::user();

            if (!$user) {
                return response()->json([
                    'success' => false,
                    'message' => 'User not authenticated'
                ], 401);
            }

            return response()->json([
                'success' => true,
                'user' => [
                    'id' => $user->id,
                    'name' => $user->name,
                    'email' => $user->email,
                    'phone_number' => $user->phone_number,
                    'balance' => $user->balance,
                    'latitude' => $user->latitude,
                    'longitude' => $user->longitude,
                    'vehicle_type' => $user->vehicle_type,
                    'vehicle_plate' => $user->vehicle_plate,
                    'orders' => $user->orders,
                    'profile_picture' => $user->profile_picture,
                ]
            ], 200);

        } catch (\Exception $e) {
            return response()->json([
                'success' => false,
                'message' => 'An error occurred: ' . $e->getMessage()
            ], 500);
        }
    }

    public function getAllDeliveries(Request $request)
    {
        try {
            $deliveries = Delivery::all()->map(function ($delivery) {
                $delivery->token = $delivery->createToken('api-token')->plainTextToken;
                return $delivery;
            });
    
            return response()->json([
                'success' => true,
                'data' => $deliveries
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'success' => false,
                'message' => 'An error occurred: ' . $e->getMessage()
            ], 500);
        }
    }

    public function logout(Request $request)
    {
        $user = Auth::user();

        if ($user && $request->bearerToken()) {
            $token = $request->bearerToken();
            $tokenData = PersonalAccessToken::findToken($token);

            if ($tokenData) {
                $tokenData->delete();
                return response()->json(['success' => true, 'message' => 'Logged out successfully']);
            } else {
                return response()->json(['success' => false, 'message' => 'Token not found'], 404);
            }
        }

        return response()->json(['success' => false, 'message' => 'User not authenticated'], 401);
    }

    public function updateLocation(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'latitude' => 'required|numeric',
            'longitude' => 'required|numeric',
        ]);

        if ($validator->fails()) {
            return response()->json(['success' => false, 'message' => $validator->errors()], 422);
        }

        $user = Auth::user();

        if ($user) {
            $user->latitude = $request->latitude;
            $user->longitude = $request->longitude;
            $user->save();

            DeliveryLocation::create([
                'delivery_id' => $user->id,
                'latitude' => $request->latitude,
                'longitude' => $request->longitude,
            ]);

            return response()->json(['success' => true, 'message' => 'Location updated successfully']);
        }

        return response()->json(['success' => false, 'message' => 'User not authenticated'], 401);
    }

    public function findNearestDelivery(Request $request)
    {
        $latitude = $request->input('latitude');
        $longitude = $request->input('longitude');

        $nearestDeliveries = DB::table('delivery_locations')
            ->select('delivery_id', DB::raw("6371 * acos(cos(radians($latitude))
                     * cos(radians(latitude))
                     * cos(radians(longitude) - radians($longitude))
                     + sin(radians($latitude))
                     * sin(radians(latitude))) AS distance"))
            ->having('distance', '<', 10)
            ->orderBy('distance')
            ->get();

        if ($nearestDeliveries->isEmpty()) {
            return response()->json(['success' => false, 'message' => 'No available deliveries within 10 km radius']);
        }

        $deliveryDetails = Delivery::whereIn('id', $nearestDeliveries->pluck('delivery_id'))->get();

        return response()->json(['success' => true, 'deliveries' => $deliveryDetails]);
    }

    public function requestDelivery(Request $request, $orderId)
    {
        Log::info('Test log message');
        try {
            Log::info('Starting requestDelivery function'); // إضافة سجل لبدء الفانكشن
            
            $order = Order::findOrFail($orderId);
            Log::info('Order found: ', ['order_id' => $order->id, 'status' => $order->status]);
    
            if ($order->status !== 'new') {
                Log::warning('Order is not in a valid state for delivery request', ['order_id' => $orderId, 'status' => $order->status]);
                return response()->json(['success' => false, 'message' => 'Order is not in a valid state for delivery request'], 400);
            }
    
            $nearbyDeliveries = Delivery::where('is_online', true)
                ->where('current_order', null)
                ->whereRaw("ST_Distance_Sphere(point(longitude, latitude), point(?, ?)) <= ?", [
                    $order->longitude, $order->latitude, 10000
                ])
                ->orderByRaw("ST_Distance_Sphere(point(longitude, latitude), point(?, ?))", [
                    $order->longitude, $order->latitude
                ])
                ->get();
            
            Log::info('Nearby deliveries found', ['count' => $nearbyDeliveries->count()]);
    
            if ($nearbyDeliveries->isEmpty()) {
                Log::warning('No available deliveries nearby or no device token');
                return response()->json(['success' => false, 'message' => 'No available deliveries nearby or no device token'], 404);
            }
    
            $delivery = $nearbyDeliveries->first();
            Log::info('Selected delivery', ['delivery_id' => $delivery->id, 'device_token' => $delivery->device_token]);
    
            if ($delivery->device_token) {
                try {
                    $firebase = (new \Kreait\Firebase\Factory)
                        ->withServiceAccount(base_path(env('FIREBASE_CREDENTIALS')));
                    
                    $messaging = $firebase->createMessaging();
                    $deviceToken = $delivery->device_token;
    
                    $message = CloudMessage::withTarget('token', $deviceToken)
                        ->withNotification(FirebaseNotification::create('New Order Request', 'You have a new delivery request for order ID: ' . $order->id));
                    
                    Log::info('Sending Firebase message', ['token' => $deviceToken]);
    
                    $response = $messaging->send($message);
                    Log::info('Firebase message sent successfully', ['response' => $response]);
    
                    Log::info('Attempting to save delivery notification', ['delivery_id' => $delivery->id]);

                    DeliveryNotification::create([
                        'delivery_id' => $delivery->id,
                        'title' => 'New Order Request',
                        'message' => 'You have a new delivery request for order ID: ' . $order->id,
                        'status' => 'new',
                        'is_read' => false
                    ]);
                    
                    Log::info('Delivery notification saved successfully');
    
                } catch (\Exception $e) {
                    Log::error('Error sending Firebase message', ['message' => $e->getMessage()]);
                    return response()->json(['success' => false, 'message' => 'An error occurred while sending the notification: ' . $e->getMessage()], 500);
                }
            } else {
                Log::warning('Delivery does not have a device token', ['delivery_id' => $delivery->id]);
            }
    
            return response()->json([
                'success' => true,
                'message' => 'Delivery request sent successfully, waiting for delivery response',
                'delivery' => $delivery,
                'order' => $order,
            ], 200);
    
        } catch (\Exception $e) {
            Log::error('An error occurred', ['message' => $e->getMessage()]);
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    
    public function getOrdersByStatus(Request $request)
    {
        try {
            $delivery = Auth::user();
    
            $status = $request->input('status');
    
            $query = Order::where('delivery_id', $delivery->id);
    
            if ($status) {
                $query->where('status', $status);
            }
    
            $orders = $query->orderBy('created_at', 'desc')->get();
    
            return response()->json([
                'success' => true,
                'orders' => $orders
            ], 200);
    
        } catch (\Exception $e) {
            return response()->json([
                'success' => false,
                'message' => 'An error occurred: ' . $e->getMessage()
            ], 500);
        }
    }


}
