<?php

namespace App\Http\Controllers;

use App\Events\LaundryCreated;
use App\Models\Laundry;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class LaundryController extends Controller
{
    public function index(Request $request)
    {
        $user = Auth::user();
        $latitude = $user->latitude;
        $longitude = $user->longitude;
    
        if (!$latitude || !$longitude) {
            return response()->json([
                'success' => false,
                'message' => 'User location is not set.'
            ], 400);
        }
    
        $laundries = DB::table('laundries')
            ->selectRaw("*, ( 6371 * acos( cos( radians(?) ) *
                          cos( radians( latitude ) )
                          * cos( radians( longitude ) - radians(?) )
                          + sin( radians(?) ) *
                          sin( radians( latitude ) ) )
                        ) AS distance",
                    [$latitude, $longitude, $latitude])
            ->orderBy('distance')
            ->get()
            ->map(function ($laundry) {
                // Convert distance to meters if less than 1 km, otherwise keep it in km
                if ($laundry->distance < 1) {
                    $laundry->distance = round(floatval($laundry->distance) * 1000); // Convert to meters
                    $laundry->unit = 'meters';
                } else {
                    $laundry->distance = round(floatval($laundry->distance), 2); // Keep it in kilometers
                    $laundry->unit = 'kilometers';
                }
                return $laundry;
            });
    
        return response()->json([
            'success' => true,
            'laundries' => $laundries,
        ]);
    }


    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'logo' => 'nullable|string|max:255',
            'latitude' => 'required|numeric',
            'longitude' => 'required|numeric',
        ]);

        $laundry = Laundry::create($request->all());

        event(new LaundryCreated($laundry));

        return response()->json($laundry, 201);
    }

    public function show(Request $request, $id)
    {
        $user = Auth::user();
        $latitude = $user->latitude;
        $longitude = $user->longitude;

        if (!$latitude || !$longitude) {
            return response()->json([
                'success' => false,
                'message' => 'User location is not set.'
            ], 400);
        }

        $laundry = DB::table('laundries')
            ->selectRaw("*, ( 6371 * acos( cos( radians(?) ) *
                              cos( radians( latitude ) )
                              * cos( radians( longitude ) - radians(?) )
                              + sin( radians(?) ) *
                              sin( radians( latitude ) ) )
                            ) AS distance",
                [$latitude, $longitude, $latitude])
            ->where('id', $id)
            ->first();

        if (!$laundry) {
            return response()->json([
                'success' => false,
                'message' => 'Laundry not found.'
            ], 404);
        }

        $distance = $laundry->distance;

        if ($distance < 1) {
            $distanceText = round($distance * 1000, 2) . ' meters';
        } else {
            $distanceText = round($distance, 2) . ' kilometers';
        }

        // Fetch service categories and services
        $categories = DB::table('service_categories')
            ->where('laundry_id', $id)
            ->get();

        foreach ($categories as $category) {
            $services = DB::table('services')
                ->where('service_category_id', $category->id)
                ->get();

            $category->services = $services;
        }

        $laundry->distance = $distanceText;
        $laundry->categories = $categories;

        return response()->json([
            'success' => true,
            'laundry' => $laundry,
        ]);
    }

    public function update(Request $request, $id)
    {
        $laundry = Laundry::findOrFail($id);

        $request->validate([
            'name' => 'sometimes|required|string|max:255',
            'logo' => 'nullable|string|max:255',
            'latitude' => 'sometimes|required|numeric',
            'longitude' => 'sometimes|required|numeric',
        ]);

        $laundry->update($request->all());

        return response()->json($laundry);
    }

    public function destroy($id)
    {
        $laundry = Laundry::findOrFail($id);
        $laundry->delete();

        return response()->json(null, 204);
    }
}
