<?php

namespace App\Http\Controllers;

use App\Models\Delivery;
use App\Models\Order;
use App\Models\OrderItem;
use App\Models\Service;
use App\Models\Notification;
use App\Models\User;
use App\Notifications\NewOrderNotification;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Kreait\Firebase\Messaging\CloudMessage;
use Kreait\Firebase\Messaging\Notification as FirebaseNotification;
use Kreait\Firebase\Factory;

class OrderController extends Controller
{
    public function createOrder(Request $request)
    {
        try {
            $user = Auth::user();

            if (!$user) {
                return response()->json(['success' => false, 'message' => 'User not authenticated'], 401);
            }

            $validator = Validator::make($request->all(), [
                'laundry_id' => 'required|exists:laundries,id',
                'delivery_address' => 'required|string|max:255',
                'latitude' => 'required|numeric',
                'longitude' => 'required|numeric',
                'delivery_type' => 'required|in:pickup,delivery',
                'payment_method' => 'required|in:google_pay,apple_pay,cash',
                'services' => 'required|array',
                'services.*.service_id' => 'required|exists:services,id',
                'services.*.quantity' => 'required|integer|min:1',
            ]);

            if ($validator->fails()) {
                return response()->json(['success' => false, 'message' => $validator->errors()], 422);
            }

            $order = Order::create([
                'user_id' => $user->id,
                'laundry_id' => $request->laundry_id,
                'delivery_address' => $request->delivery_address,
                'latitude' => $request->latitude,
                'longitude' => $request->longitude,
                'delivery_type' => $request->delivery_type,
                'payment_method' => $request->payment_method,
                'payment_status' => 'unpaid',
                'status' => 'new',
                'total' => 0,
            ]);

            $total = 0;

            foreach ($request->services as $service) {
                $serviceData = Service::find($service['service_id']);
                if (!$serviceData) {
                    return response()->json(['success' => false, 'message' => 'Service not found'], 404);
                }

                $price = $serviceData->price * $service['quantity'];
                $total += $price;

                OrderItem::create([
                    'order_id' => $order->id,
                    'service_id' => $service['service_id'],
                    'quantity' => $service['quantity'],
                    'price' => $price,
                ]);
            }

            $order->total = $total;
            $order->save();
            
            if ($user->device_token) {
                try {
                    $firebase = (new \Kreait\Firebase\Factory)
                        ->withServiceAccount(base_path(env('FIREBASE_CREDENTIALS')));
                
                    $messaging = $firebase->createMessaging();
                    $deviceToken = $user->device_token;
                
                    $message = CloudMessage::withTarget('token', $deviceToken)
                        ->withNotification(FirebaseNotification::create('New Order Created', 'Your order with ID: ' . $order->id . ' has been created.'));
                
                    $messaging->send($message);
                
                    Notification::create([
                        'user_id' => $user->id,
                        'title' => 'New Order Created',
                        'message' => 'Your order with ID: ' . $order->id . ' has been created.',
                        'status' => 'new',
                    ]);
                
                    Log::info('Notification sent successfully to user ID: ' . $user->id);
                
                } catch (\Kreait\Firebase\Exception\Messaging\NotFound $e) {
                    Log::error('Device token not found or invalid: ' . $user->device_token);
                } catch (\Kreait\Firebase\Exception\MessagingException $e) {
                    Log::error('Error sending notification: ' . $e->getMessage());
                }
            }

            return response()->json([
                'success' => true,
                'order' => $order->load('items.service'),
            ], 201);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function getUserOrders(Request $request)
    {
        $user = Auth::user();

        $orders = Order::where('user_id', $user->id)->with(['laundry', 'items.service'])->get();

        if (!$orders) {
            return response()->json(['success' => false, 'message' => 'No Orders for this user'], 404);
        }

        return response()->json([
            'success' => true,
            'orders' => $orders,
        ]);
    }

    public function showOrderDetails($id)
    {
        $order = Order::with(['user', 'laundry', 'items.service'])->find($id);

        if (!$order) {
            return response()->json(['success' => false, 'message' => 'Order not found'], 404);
        }

        return response()->json([
            'success' => true,
            'order' => $order,
        ]);
    }

    public function requestDelivery(Request $request, $orderId)
    {
        try {
            $order = Order::findOrFail($orderId);

            if ($order->status !== 'new') {
                return response()->json(['success' => false, 'message' => 'Order is not in a valid state for delivery request'], 400);
            }

            $latitude = $request->input('latitude');
            $longitude = $request->input('longitude');

            $nearbyDeliveries = DB::table('delivery_locations')
                ->select('delivery_id', DB::raw("6371 * acos(cos(radians($latitude))
                     * cos(radians(latitude))
                     * cos(radians(longitude) - radians($longitude))
                     + sin(radians($latitude))
                     * sin(radians(latitude))) AS distance"))
                ->having('distance', '<', 10)
                ->orderBy('distance')
                ->get();

            if ($nearbyDeliveries->isEmpty()) {
                return response()->json(['success' => false, 'message' => 'No available deliveries nearby'], 404);
            }

            $delivery = $nearbyDeliveries->first();

//            if($delivery->device_token) {
//                try {
//                    $firebase = (new \Kreait\Firebase\Factory)
//                        ->withServiceAccount(base_path(env('FIREBASE_CREDENTIALS')));
//
//                    $messaging = $firebase->createMessaging();
//                    $deviceToken = $delivery->device_token;
//
//                    $message = CloudMessage::withTarget('token', $deviceToken)
//                        ->withNotification(FirebaseNotification::create('New Order Request', 'You have a new delivery request.'));
//
//                    $messaging->send($message);
//
//                    Log::info('Delivery request sent to delivery ID: ' . $delivery->id);
//                } catch (\Exception $e) {
//                    Log::error('Error sending delivery request notification: ' . $e->getMessage());
//                }
//            }

            return response()->json([
                'success' => true,
                'message' => 'Delivery request sent successfully, waiting for delivery response',
                'delivery' => $delivery,
                'order' => $order,
            ], 200);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function markInProgress($orderId)
    {
        try {
            $order = Order::findOrFail($orderId);

            if ($order->status !== 'on_pick_up') {
                return response()->json(['success' => false, 'message' => 'Order is not in a valid state for in-progress'], 400);
            }

            $order->status = 'in_progress';
            $order->save();

            // Send notification to the user
            $user = $order->user;
            if ($user->device_token) {
                // Code for sending notification
            }

            return response()->json(['success' => true, 'message' => 'Order marked as in progress', 'order' => $order], 200);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function markReady($orderId)
    {
        try {
            $order = Order::findOrFail($orderId);

            if ($order->status !== 'in_progress') {
                return response()->json(['success' => false, 'message' => 'Order is not in a valid state for marking ready'], 400);
            }

            $order->status = 'ready';
            $order->save();

            // Send notification to the user
            $user = $order->user;
            if ($user->device_token) {
                // Code for sending notification
            }

            return response()->json(['success' => true, 'message' => 'Order marked as ready', 'order' => $order], 200);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function choosePickupType(Request $request, $orderId)
    {
        $validator = Validator::make($request->all(), [
            'pickup_type' => 'required|in:user_pick_up,delivery_to_user'
        ]);

        if ($validator->fails()) {
            return response()->json(['success' => false, 'message' => $validator->errors()], 422);
        }

        try {
            $order = Order::findOrFail($orderId);

            if ($order->status !== 'ready') {
                return response()->json(['success' => false, 'message' => 'Order is not in a valid state for pickup choice'], 400);
            }

            $order->status = $request->pickup_type;
            $order->save();

            return response()->json(['success' => true, 'message' => 'Pickup type chosen', 'order' => $order], 200);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function markOnDelivery($orderId)
    {
        try {
            $order = Order::findOrFail($orderId);

            if ($order->status !== 'ready') {
                return response()->json(['success' => false, 'message' => 'Order is not ready for delivery'], 400);
            }

            $order->status = 'on_delivery';
            $order->save();

            return response()->json(['success' => true, 'message' => 'Order marked as on delivery', 'order' => $order], 200);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

    public function markCompleted($orderId)
    {
        try {
            $order = Order::findOrFail($orderId);

            if ($order->status !== 'on_delivery') {
                return response()->json(['success' => false, 'message' => 'Order is not in delivery state'], 400);
            }

            $order->status = 'completed';
            $order->save();

            return response()->json(['success' => true, 'message' => 'Order completed', 'order' => $order], 200);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }


    public function cancelOrder(Request $request, $orderId)
    {
        try {
            $user = Auth::user();

            $order = Order::where('id', $orderId)
                ->where('user_id', $user->id)
                ->first();

            if (!$order) {
                return response()->json(['success' => false, 'message' => 'Order not found or you are not authorized to cancel it'], 404);
            }

            if ($order->status === 'completed' || $order->status === 'canceled') {
                return response()->json(['success' => false, 'message' => 'This order cannot be canceled'], 400);
            }

            $order->status = 'canceled';
            $order->save();

            return response()->json([
                'success' => true,
                'message' => 'Order has been canceled successfully',
                'order' => $order,
            ], 200);

        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()], 500);
        }
    }

}
