<?php

namespace App\Notifications;

use App\Models\Order;
use App\Models\User;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Notification;
use Illuminate\Notifications\Messages\BroadcastMessage;
use Illuminate\Support\Facades\Log;
use Kreait\Firebase\Messaging\CloudMessage;
use Kreait\Firebase\Messaging\Notification as FirebaseNotification;

class NewOrderNotification extends Notification
{
    use Queueable;


    protected $order;

    /**
     * Create a new notification instance.
     *
     * @param  \App\Models\Order  $order
     * @return void
     */
    public function __construct(Order $order)
    {
        $this->order = $order;
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function via($notifiable)
    {
        return ['database', 'broadcast', \App\Notifications\Channels\FirebaseChannel::class];
    }

    /**
     * Get the array representation of the notification for storing in the database.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function toArray($notifiable)
    {
        return [
            'order_id' => $this->order->id,
            'message' => 'Your order has been successfully created!',
            'user_id' => $notifiable->id,
            'notifiable_id' => $notifiable->id,
            'notifiable_type' => get_class($notifiable),
        ];
    }

    /**
     * Get the broadcastable representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return BroadcastMessage
     */
    public function toBroadcast($notifiable)
    {
        return new BroadcastMessage([
            'order_id' => $this->order->id,
            'message' => 'Your order has been successfully created!',
        ]);
    }

    /**
     * Send the notification via Firebase Cloud Messaging.
     *
     * @param  mixed  $notifiable
     * @return \Kreait\Firebase\Messaging\CloudMessage
     */
    public function toFirebase($notifiable)
    {
        $firebase = (new \Kreait\Firebase\Factory)
            ->withServiceAccount(base_path(env('FIREBASE_CREDENTIALS')));

        $messaging = $firebase->createMessaging();
        $deviceToken = $notifiable->device_token; // استخدام الـ device token الخاص بالمستخدم

        $message = CloudMessage::withTarget('token', $deviceToken)
            ->withNotification(FirebaseNotification::create('New Order Created', 'Your order with ID: ' . $this->order->id . ' has been created.'));

        try {
            $messaging->send($message);
        } catch (\Exception $e) {
            Log::error('Error sending notification: ' . $e->getMessage());
        }
    }

    /**
     * Get the array representation of the notification for Firebase.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function toFirebaseArray($notifiable)
    {
        return [
            'notification' => [
                'title' => 'New Order Created',
                'body' => 'Your order with ID: ' . $this->order->id . ' has been created.',
            ],
            'data' => [
                'order_id' => $this->order->id,
            ],
        ];
    }
}
